function results=pantelides(X,Graph,E)

% PANTELIDES  Pantelides algorithm
%
%   PANTELIDES(X,G,E) runs pantelides algorithm on the structural model
%   described by variables X and biadjacency matrix G corresponding to
%   equations E.
%
%   Inputs:
%     X  - Cell array describing the variables in the model.
%          For example:
%            X = {{'x',0},{'x',1},{y,0}}
%          corresponds to the variables {x, x', y}. Note that
%          the model must be written in a form where no derivative 
%          is higher than 1. Also, for all variables x, {'x',0} has
%          to be included.
%
%    G  - Biadjacency matrix for the graph representing the 
%         structural model
%    
%    E  - Names of equations (optional). For example E={'e1','e2','e3'}
%
%  Outputs:
%    Structure with the following members
%
%      E       = equation names
%      X       = unknown variable names
%      graph   = matrix representing the edges of the bipartite graph with
%                equations E and variables X as node sets.
%      str_idx = structural index of the underlying DAE
%      assign  = partial assignment. This is a perfect matching between
%                the most differentiated equations in E and the highest
%                derivatives in X.
%      nu      = minimal times each equation has to be differentiated
%                to determine consistent initial conditions

% Author: Mattias Krysander
% Updated by Erik Frisk

if nargin < 2 | nargin > 3
    error('Wrong number of input arguments.')
end

neq = size(Graph,1);

if nargin == 2
    for i=1:neq
        E{i} = strcat('e',num2str(i));
    end
end
 
%initializing
A=zeros(1,length(X));
remaining = 1:length(X);

%Creating variable assosiation vector A
while ~isempty(remaining)
    
    Xtype = remaining(1);
    remaining = remaining(2:end);
    
    for j=remaining
        if strcmp(X{Xtype(1)}{1},X{j}{1})
            Xtype = [Xtype j];
            remaining = setdiff(remaining,j);
        end
    end
    
    if length(Xtype)>2
        error('Only two enteries per variable type is allowed.')
    elseif length(Xtype)==2
        tmp = [X{Xtype(1)}{2} X{Xtype(2)}{2}];
        
        idx2zero = find(tmp == 0);
        if isempty(idx2zero)
            error('The non-derivative must be included in the model.')
        else
            idx2one = find(tmp == 1);
            if isempty(idx2one)
                error('Higher derivaties than 1 are not allowed.')
            else
                A(Xtype(idx2zero)) = Xtype(idx2one);
            end
        end
    else
        if X{Xtype}{2} ~= 0
            error('The non-derivative must be included in the model.')
        end
    end
end
[A,B,Graph,assign]=findMSS(Graph,A);

%Computing structural index and equation derivatives.

derivatives = zeros(size(B));
for i=1:length(B)
    if B(i) ~= 0
        derivatives(B(i)) = derivatives(i) + 1;
        E{B(i)} = strcat(E{i},char(39));
    end
end
results.E = E;

nu = zeros(neq,1);
for k=1:length(nu)
    nu(k)=dnum(B,k);
end
results.nu=nu;

for i=1:length(A)
    if A(i) ~= 0
        X{A(i)} = {X{i}{1}, X{i}{2} + 1};
    end
end
idx1problem = 0;
for i=find(A==0)
    if X{i}{2} == 0
        idx1problem = 1;
    end
end

results.str_idx = max(derivatives) + idx1problem;
results.X = X;
results.graph = Graph;
results.assign = assign;

%printPantelides(results,'results.txt');

function n=dnum(B,i)
% Converts the B-vector into the maximum number of times a particular
% equations is differentiated
if B(i)==0
    n=0;
else
    n=1+dnum(B,B(i));
end


